#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import logging

from cdumay_opentracing import OpenTracingSpan, OpenTracingManager
from kser.schemas import Message
from kser.sequencing.operation import Operation

logger = logging.getLogger(__name__)


class OpentracingOperation(Operation):
    def _prerun(self):
        """ To execute before running message

        :return: Kafka message
        :rtype: kser.schemas.Message
        """
        logger.debug(self.label("PreRun"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))

        with OpenTracingSpan(self, self.label("PreRun")) as span:
            self.check_required_params()
            self._set_status("RUNNING")
            out = self.prerun()
            OpenTracingManager.log_kv(span, self, "done")
            return out

    def _postrun(self, result):
        """ To execute after exection

        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.debug(self.label("PostRun"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))

        with OpenTracingSpan(self, self.label("PostRun")) as span:
            out = self.postrun(result)
            OpenTracingManager.log_kv(span, self, "done")
            return out

    def unsafe_execute(self, result=None):
        with OpenTracingSpan(self, self.label("Execute")) as span:
            span.log_kv(dict(event="starting..."))
            self._prerun()
            for task in self.tasks:
                result = task.unsafe_execute(result=result)
                if result.retcode != 0:
                    return self._onerror(result)

            result = self._onsuccess(self._postrun(result=result))
            OpenTracingManager.log_kv(span, self, "done")
            return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""

import logging

from cdumay_opentracing import OpenTracingSpan, OpenTracingManager
from kser import KSER_METRICS_ENABLED, KSER_TASK_COUNT
from kser.schemas import Message
from kser.sequencing.task import Task

logger = logging.getLogger(__name__)


class OpentracingTask(Task):
    def _prerun(self):
        """ To execute before running message

        :return: Kafka message
        :rtype: kser.schemas.Message
        """
        logger.debug(self.label("PreRun"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))
        with OpenTracingSpan(self, self.label("PreRun")) as span:
            self.check_required_params()
            out = self.prerun()
            OpenTracingManager.log_kv(span, self, "done")
            return out

    def _run(self):
        """ Execution body

        :return: Execution result
        :rtype: kser.result.Result
        """
        if KSER_METRICS_ENABLED == "yes":
            KSER_TASK_COUNT.inc()

        logger.debug(self.label("Run"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))

        with OpenTracingSpan(self, self.label("Run")) as span:
            out = self.run()
            OpenTracingManager.log_kv(span, self, "done")
            return out

    def _postrun(self, result):
        """ To execute after exection

        :param kser.result.Result result: Execution result
        :return: Execution result
        :rtype: kser.result.Result
        """
        logger.debug(self.label("PostRun"), extra=dict(kmsg=Message(
            self.uuid, entrypoint=self.__class__.path,
            params=self.params, metadata=self.metadata
        ).dump()))

        with OpenTracingSpan(self, self.label("PostRun")) as span:
            out = self.postrun(result)
            OpenTracingManager.log_kv(span, self, "done")
            return out

    def unsafe_execute(self, result=None):
        """ un-wrapped execution, can raise excepetion

        :return: Execution result
        :rtype: kser.result.Result
        """
        if result:
            self.result += result

        with OpenTracingSpan(self, self.label("Execute")) as span:
            self._prerun()
            out = self._onsuccess(self._postrun(self._run()))
            OpenTracingManager.log_kv(span, self, "done")
            return out



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import base64
import os
import csodium
import marshmallow

from cdumay_error import ValidationError
from marshmallow import Schema, fields
from kser.schemas import Message


class CryptoSchema(Schema):
    data = fields.String(required=True)
    nonce = fields.String(required=True)

    def encode(self, kmsg):
        """ Encode message using libsodium

        :param kser.schemas.Message kmsg: Kafka message
        :return: the Encoded message
        """
        nonce = csodium.randombytes(csodium.crypto_box_NONCEBYTES)
        return self.dumps(dict(
            nonce=base64.encodebytes(nonce).strip(),
            data=base64.encodebytes(
                csodium.crypto_secretbox(
                    bytes(kmsg.MARSHMALLOW_SCHEMA.dumps(kmsg).data, 'utf-8'),
                    nonce, base64.b64decode(self.context['secretbox_key'])
                )
            ).strip()
        ))

    def decode(self, jdata):
        """ Decode message using libsodium

        :param str jdata: jdata to load
        :return: the Encoded message
        """
        ckmsg = self.loads(jdata)
        return Message.loads(
            csodium.crypto_secretbox_open(
                base64.b64decode(ckmsg["data"]),
                base64.b64decode(ckmsg["nonce"]),
                base64.b64decode(self.context['secretbox_key'])
            ).decode('utf-8')
        )


class CryptoMessage(Message):
    MARSHMALLOW_SCHEMA = CryptoSchema(
        context=dict(secretbox_key=os.getenv("KSER_SECRETBOX_KEY", None))
    )

    @classmethod
    def loads(cls, json_data):
        """description of load"""
        try:
            return cls(**cls.MARSHMALLOW_SCHEMA.decode(json_data))
        except marshmallow.exceptions.ValidationError as exc:
            raise ValidationError("Failed to load message", extra=exc.args[0])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. codeauthor:: Cédric Dumay <cedric.dumay@gmail.com>


"""
import opentracing
from cdumay_opentracing import OpenTracingDriver
from cdumay_result import ResultSchema


class OpenTracingKserDriver(OpenTracingDriver):
    @classmethod
    def extract(cls, data):
        trace = data.metadata.get('__parent-span__', dict())
        if len(trace) > 0:
            return opentracing.tracer.extract(cls.FORMAT, data.trace)

    @classmethod
    def inject(cls, span, data):
        data.metadata['__parent-span__'] = dict()
        opentracing.tracer.inject(
            span, cls.FORMAT, data.metadata['__parent-span__']
        )

    @classmethod
    def tags(cls, data):
        return dict(uuid=data.uuid, entrypoint=data.__class__.path)

    @classmethod
    def log_kv(cls, span, data, event, **kwargs):
        return span.log_kv({
            "event": event, **ResultSchema().dump(data.result), **kwargs
        })
